#ifndef PROCESSING_H
#define PROCESSING_H

// Functions for processing the received data

#include "main.h"


#define PREAMP_FACTOR 10.0f
#define CT_VOLTS_PER_AMP 0.097f // we're using the value at 100R load, 1A current
#define MV_RANGE 3300.0f
#define DIGITAL_RANGE 4095.0f


/**
 * @brief Comvert raw ADC samples to milliamps.
 * @param floats
 * @param uints
 * @param count
 */
void samples_to_milliamps(float *floats, const uint32_t *uints, uint16_t count);

enum FFT_LEN {
	FFT_16 = 16,
	FFT_32 = 32,
	FFT_64 = 64,
	FFT_128 = 128,
	FFT_256 = 256,
	FFT_512 = 512,
	FFT_1024 = 1024,
	FFT_2048 = 2048,
//	FFT_4096 = 4096,
};

// actually number of samples, not bins - bins = this / 2
#define IS_FFT_LEN(x) (((x) & ((x) - 1)) == 0 && (x) >= 16 && (x) <= 2048)

/**
 * @brief Take FFT of the samples captured
 *
 * The FFT will have 1/2 the length
 *
 * @param bins    : target bins (can be the same array as samples)
 * @param samples : samples
 * @param length  : sample count
 */
void samples_to_fft(float *bins, const float *samples, uint16_t length);

#endif // PROCESSING_H
