#include "gpio.h"
#include "utils/timebase.h"

void gpio_conf_output(GPIO_TypeDef* GPIOx, GPIOSpeed_TypeDef GPIO_Speed, GPIOOType_TypeDef GPIO_OType, uint16_t pins)
{
	GPIO_InitTypeDef gpio_cfg;

	gpio_cfg.GPIO_Pin = pins;
	gpio_cfg.GPIO_Mode = GPIO_Mode_OUT;
	gpio_cfg.GPIO_Speed = GPIO_Speed;
	gpio_cfg.GPIO_OType = GPIO_OType;
	gpio_cfg.GPIO_PuPd = GPIO_PuPd_NOPULL; // N/A for output

	GPIO_Init(GPIOx, &gpio_cfg);
}


void gpio_conf_input(GPIO_TypeDef* GPIOx, GPIOPuPd_TypeDef GPIO_PuPd, uint16_t pins)
{
	GPIO_InitTypeDef gpio_cfg;

	gpio_cfg.GPIO_Pin  = pins;
	gpio_cfg.GPIO_Mode = GPIO_Mode_IN;
	gpio_cfg.GPIO_Speed = GPIO_Speed_2MHz; // N/A for input
	gpio_cfg.GPIO_OType = GPIO_OType_PP;   // N/A for input
	gpio_cfg.GPIO_PuPd = GPIO_PuPd;

	GPIO_Init(GPIOx, &gpio_cfg);
}


void gpio_conf_analog(GPIO_TypeDef* GPIOx, uint16_t pins)
{
	GPIO_InitTypeDef gpio_cfg;

	gpio_cfg.GPIO_Pin = pins;
	gpio_cfg.GPIO_Mode = GPIO_Mode_AN;
	gpio_cfg.GPIO_Speed = GPIO_Speed_2MHz; // N/A for analog
	gpio_cfg.GPIO_OType = GPIO_OType_PP;   // N/A for analog
	gpio_cfg.GPIO_PuPd = GPIO_PuPd_NOPULL;

	GPIO_Init(GPIOx, &gpio_cfg);
}


void gpio_conf_af(GPIO_TypeDef* GPIOx, GPIOSpeed_TypeDef speed, GPIOOType_TypeDef otype, uint8_t af, uint16_t pins)
{
	GPIO_InitTypeDef gpio_cfg;

	gpio_cfg.GPIO_Pin = pins;
	gpio_cfg.GPIO_Mode = GPIO_Mode_AF;
	gpio_cfg.GPIO_Speed = speed;
	gpio_cfg.GPIO_OType = otype;
	gpio_cfg.GPIO_PuPd = GPIO_PuPd_NOPULL;

	GPIO_Init(GPIOx, &gpio_cfg);

	// configure AF for the pins
	for (uint16_t i = 0; i < 16; i++) {
		if (pins & (uint16_t)(1 << i)) {
			GPIO_PinAFConfig(GPIOx, i, af);
		}
	}
}

// ---------------------------

void esp_reset(bool nrst_assert)
{
	if (nrst_assert) {
		// reset line DOWN
		GPIOA->BRR |= GPIO1;
	} else {
		// reset line UP
		GPIOA->BSRR |= GPIO1;
	}
}
