#include "watchdog.h"
#include "reset.h"
#include "com/debug.h"

bool WDG_Configure(ms_time_t period)
{
	IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);

	// Calculate the prescaller and preload value

	const uint32_t clock = 40000;
	uint8_t presc = 0; // 0 .. 6
	uint32_t preload = 0;
	bool suc = false;

	for (presc = IWDG_Prescaler_4; presc <= IWDG_Prescaler_256; presc++) {
		preload = ((period * clock) / ((0x04U << presc) * 1000));

		if (preload < 0xFFF) {
			suc = true;
			break;
		}
	}

	if (!suc) {
		// default - 5s
		presc = IWDG_Prescaler_64;
		preload = 0xC35;
	}

	// Set to hardware

	IWDG_SetReload((uint16_t) preload);
	IWDG_SetPrescaler(presc);

	IWDG_Enable();

	return suc;
}


// ---- quick & dirty soft WDG ----

#define swdg_acq_idle_reload 5000 // restart after 5 s
static volatile uint32_t swdg_acq_idle = swdg_acq_idle_reload;


void SoftWDG_Feed(SoftWdgGroup group)
{
	switch (group) {
		case SoftWDG_ACQ_IDLE: swdg_acq_idle = swdg_acq_idle_reload; break;
	}
}

// every ms
void SoftWDG_Tick1ms(void *unused)
{
	(void)unused;

	if (--swdg_acq_idle == 0) {
		error("SoftWDG \"acq idle\" timeout, restarting.");
		SW_Reset();
		return;
	}
}
