#pragma once
#include "main.h"

/** Enum for DMA data width. Will be converted to bitmasks by the acq routine. */
enum DMADataWidth {
	DMADataWidth_8 = 8,
	DMADataWidth_16 = 16,
	DMADataWidth_32 = 32
};


/**
 * @brief Configure peripherals for capturing analog data
 *
 * - ADC1
 * - DMA1 channel 1
 * - TIM1
 */
void acq_init(void);


/**
 * @brief Read current value of ADC1_IN1
 *
 * This function reads a single value, and is not optimized for speed.
 *
 * @param dest : pointer to a target variable
 * @return success
 */
bool acq_one_sample(uint16_t *dest);


/**
 * @brief Start DMA from the ADC.
 *
 * Completion will be announced via the event queue.
 *
 * @param target : target array
 * @param count  : array length
 * @param frequency : Hz sampling frequency
 * @param width : data width of the buffer
 */
bool acq_start_dma_capture(void *target, uint16_t count, uint32_t frequency, enum DMADataWidth width);


/**
 * @brief Check if the batch capture using DMA is in progress
 * @return DMA running
 */
bool acq_dma_in_progress(void);
