#include "adc.h"
#include "utils/timebase.h"
#include "com/debug.h"


/** Turn the ADC on if turned off. */
bool adc_on(ADC_TypeDef *ADCx)
{
	if (ADCx->CR & ADC_CR_ADEN) {
		return true; // already ON
	}

	ADCx->CR |= ADC_CR_ADEN; // turn ADC on

	// wait for confirmation
	until_timeout(10) {
		if(ADCx->ISR & ADC_ISR_ADRD) {
			return true;
		}
	}

	error("ADC enable failed.");
	return false;
}


/** Turn the ADC off if turned on */
bool adc_off(ADC_TypeDef *ADCx)
{
	if (!(ADCx->CR & ADC_CR_ADEN)) {
		return true; // already off
	}

	ADCx->CR |= ADC_CR_ADDIS; // turn ADC off

	// wait for ADEN to go down
	until_timeout(10) {
		if(!(ADCx->CR & ADC_CR_ADEN)) {
			return true;
		}
	}

	error("ADC disable failed.");
	return false;
}


bool adc_calibrate(ADC_TypeDef *ADCx)
{
	// make sure conv is stopped
	ADCx->CR &= ~ADC_CR_ADSTART;
	ADCx->CR |= ADC_CR_ADSTP;

	// Disable ADC
	adc_off(ADCx);

	// Calibrate
	ADCx->CR |= ADC_CR_ADCAL;
	bool suc = false;
	until_timeout(10) {
		if(! (ADCx->CR & ADC_CR_ADCAL)) {
			suc = true;
			break;
		}
	}

	if(!suc) dbg("adc calib timeout");

	// Enable ADC
	adc_on(ADCx);

	return suc;
}
