#include "mutex.h"
#include "utils/timebase.h"

#define MUTEX_UNLOCKED 0
#define MUTEX_LOCKED   1

#define STREX_SUCCESS 0
#define STREX_FAIL 1


/**
 * @brief Try to lock.
 * @param lock : the mutex to lock
 * @return 1 if the lock was acquired
 */
static bool mutex_trylock(mutex_t *lock)
{
	bool suc = false;

	/* If the mutex is unlocked. */
	if (__LDREXW(lock) == MUTEX_UNLOCKED) {
		/* Try to lock it. */
		suc = (STREX_SUCCESS == __STREXW(MUTEX_LOCKED, lock)); // 1...failed, 0...succeeded
	}

	__DMB(); // magic barrier instruction

	return suc;
}


/** Claim */
bool mutex_lock(mutex_t *lock, ms_time_t timeout)
{
	if (timeout == 0) {
		// single lock attempt, no waiting
		return mutex_trylock(lock);
	}

	until_timeout(timeout) {
		if (mutex_trylock(lock)) {
			return true;
		}
	}

	return false;
}


/** Release */
void mutex_unlock(mutex_t *lock)
{
	/* Ensure accesses to protected resource are finished */
	__DMB();

	/* Free the lock. */
	*lock = MUTEX_UNLOCKED;
}
